import os

from tools import ner_predict_utils
from tools import tag_predict_utils
from tools import sentiment_predict_utils


class train(object):
    def __init__(self,task_name,data_dir,model_dir,output_dir,
                 label_dict='label2id.pkl',label_list=None,
                 eval=True,max_seq_length=128,
                 learning_rate=2e-5,batch_size=32):
        '''
        训练函数
        :param task_name:任务名：目前包括实体识别ner，序列标注tag，句子分类classify，句子配对pair
        :param label_list:任务的标签序列
            1. ner，tag,classify 都需要这个参数
            2. 在序列标注任务中要加入【CLS】,[SEP]
        :param label_dict: 序列标注任务中标签与ID对应的字典名
        :param data_dir: 数据文件
        :param model_dir: 模型文件
        :param output_dir: 输出文件
        :param eval: 是否进行验证
        :param max_seq_length:
        :param learning_rate:
        :param batch_size:
        '''
        self.task_name=task_name
        if label_list:
            self.label_list=','.join(label_list)
        else:
            self.label_list=None
        self.label_dict=label_dict
        self.data_dir=os.path.abspath(data_dir)
        model_dir1=os.path.abspath(model_dir)
        self.model_dir=os.path.join(model_dir1,'bert_model.ckpt')

        self.work_path=os.getcwd()

        self.output_dir=os.path.abspath(output_dir)
        self.eval=eval
        self.max_seq_length=max_seq_length
        self.lr=learning_rate
        self.batch_size=batch_size

        self.module_path = os.path.dirname(__file__)

    def __call__(self):
        os.chdir(self.module_path)
        try:
            self._train()
        finally:
            os.chdir(self.work_path)

    def _train(self):

       if self.eval:
           os.system('python run_classifier.py \
                      --label_dict={} \
                      --label_list={} \
                      --task_name={} \
                      --do_train=true \
                      --do_eval=true \
                      --data_dir={} \
                      --vocab_file=./vocab.txt \
                      --bert_config_file=./bert_config.json \
                      --init_checkpoint={} \
                      --max_seq_length={} \
                      --train_batch_size={} \
                      --learning_rate={} \
                      --output_dir={}'.format(self.label_dict,
                                              self.label_list,
                                              self.task_name,
                                              self.data_dir,
                                              self.model_dir,
                                              self.max_seq_length,
                                              self.batch_size,
                                              self.lr,
                                              self.output_dir))
       else:
           os.system('python run_classifier.py \
                                 --label_dict={} \
                                 --label_list={} \
                                 --task_name={} \
                                 --do_train=true \
                                 --data_dir={} \
                                 --vocab_file=./vocab.txt \
                                 --bert_config_file=./bert_config.json \
                                 --init_checkpoint={} \
                                 --max_seq_length={} \
                                 --train_batch_size={} \
                                 --learning_rate={} \
                                 --output_dir={}'.format(self.label_dict,
                                                         self.label_list,
                                                         self.task_name,
                                                         self.data_dir,
                                                         self.model_dir,
                                                         self.max_seq_length,
                                                         self.batch_size,
                                                         self.lr,
                                                         self.output_dir))

    def _predict(self):
        os.system('python run_classifier.py \
                                         --label_dict={} \
                                         --label_list={} \
                                         --task_name={} \
                                         --do_predict=true \
                                         --data_dir={} \
                                         --vocab_file=./vocab.txt \
                                         --bert_config_file=./bert_config.json \
                                         --init_checkpoint={} \
                                         --max_seq_length={} \
                                         --train_batch_size={} \
                                         --learning_rate={} \
                                         --output_dir={}'.format(self.label_dict,
                                                                 self.label_list,
                                                                 self.task_name,
                                                                 self.data_dir,
                                                                 self.model_dir,
                                                                 self.max_seq_length,
                                                                 self.batch_size,
                                                                 self.lr,
                                                                 self.output_dir))

    @ staticmethod
    def help():
        print('''训练函数——参数解析：
        :param task_name:任务名：目前包括实体识别ner，序列标注tag，句子分类classify，句子配对pair
        :param label_list: 任务的标签列表，在序列标注任务中要加入【CLS】,[SEP]
        :param label_dict: 序列标注任务中标签与ID对应的字典名
        :param data_dir: 数据文件
        :param model_dir: 模型文件
        :param output_dir: 输出文件
        :param eval: 是否进行验证
        :param max_seq_length:
        :param learning_rate:
        :param batch_size:
        ''')
        print('''文件格式说明：
        1. 序列标注任务文件格式为 word tag
        2. 文本分类任务文件格式为 sentence label
        3. 句子配对任务文件格式为 index text1 text2 label  ，其中index为不必要的列，中间分隔符为\t
        ''')

    def predict(self):
        os.chdir(self.module_path)
        try:
            self._predict()

            if self.task_name == 'ner' :

                text_file = os.path.join(self.output_dir,'token_test.txt')
                label_file = os.path.join(self.output_dir,'label_test.txt')

                texts = ner_predict_utils.get_data(text_file)
                labels = ner_predict_utils.get_data(label_file)
                texts = ner_predict_utils.proecess_data(texts)
                labels = ner_predict_utils.proecess_data(labels)

                result = ner_predict_utils.recognized_entity(texts, labels)
                return result

            if self.task_name =='tag':
                text_file = os.path.join(self.output_dir, 'token_test.txt')
                label_file = os.path.join(self.output_dir, 'label_test.txt')

                texts = tag_predict_utils.get_data(text_file)
                label = tag_predict_utils.get_data(label_file)
                texts = tag_predict_utils.proecess_data(texts)
                labels = tag_predict_utils.proecess_data(label)

                cut, posseg = tag_predict_utils.segmentor(texts, labels)

                return cut,posseg

            if self.task_name == 'classify':
                lines = sentiment_predict_utils.get_data(os.path.join(self.output_dir,'test_results.tsv'))
                lines = sentiment_predict_utils.get_result(lines)
                results = sentiment_predict_utils.output(lines,True)
                return results

            if self.task_name == 'pair':
                lines = sentiment_predict_utils.get_data(os.path.join(self.output_dir, 'test_results.tsv'))
                lines = sentiment_predict_utils.get_result(lines)
                results = sentiment_predict_utils.output(lines, True)
                return results

        finally:
            os.chdir(self.work_path)





import os
import shutil

from tools import ner_predict_utils
from tools import tag_predict_utils
from tools import sentiment_predict_utils


class tagger(object):

    def __init__(self, model_dir):
        self._work_path = os.getcwd()
        self._module_path = os.path.dirname(__file__)
        self._model_dir = os.path.abspath(model_dir)

    def _data_process(self, document):
        try:
            os.chdir(self._module_path)
            if not os.path.exists('./output'):
                os.mkdir('./output')
            with open('./output/test.txt', 'w+', encoding='utf-8') as f:
                for sentence in document:
                    for word in sentence:
                        f.write(word + '\n')
                    f.write('\n')
        finally:
            os.chdir(self._work_path)

    def ner(self, document):
        model_path = os.path.join(self._model_dir, 'ner_model/model')
        if type(document) is str:
            raise Exception('the document must be list or tuple')
        self._data_process(document)
        try:
            os.chdir(self._module_path)
            os.system('python pos_tag.py \
                        --task_name=ner \
                        --do_predict=true \
                        --data_dir=./output \
                        --vocab_file=./vocab.txt \
                        --bert_config_file=./bert_config.json \
                        --init_checkpoint={} \
                        --max_seq_length=128 \
                        --train_batch_size=32 \
                        --learning_rate=2e-5 \
                        --output_dir=./output'.format(model_path))

            text_file = './output/token_test.txt'
            label_file = './output/label_test.txt'

            texts = ner_predict_utils.get_data(text_file)
            labels = ner_predict_utils.get_data(label_file)
            texts = ner_predict_utils.proecess_data(texts)
            labels = ner_predict_utils.proecess_data(labels)

            result = ner_predict_utils.recognized_entity(texts, labels)

            shutil.rmtree('./output')

        finally:
            os.chdir(self._work_path)

        return result

    def cut(self, document, mode='cut'):
        model_path = os.path.join(self._model_dir, 'tag_model/model')
        if type(document) is str:
            raise Exception('the document must be list or tuple')
        self._data_process(document)

        try:
            os.chdir(self._module_path)

            os.system('python pos_tag.py \
                               --task_name=tag \
                               --do_predict=true \
                               --data_dir=./output \
                               --vocab_file=./vocab.txt \
                               --bert_config_file=./bert_config.json \
                               --init_checkpoint={} \
                               --max_seq_length=128 \
                               --train_batch_size=32 \
                               --learning_rate=2e-5 \
                               --output_dir=./output'.format(model_path))

            text_file = './output/token_test.txt'
            label_file = './output/label_test.txt'
            texts = tag_predict_utils.get_data(text_file)
            label = tag_predict_utils.get_data(label_file)
            texts = tag_predict_utils.proecess_data(texts)
            labels = tag_predict_utils.proecess_data(label)

            cut, posseg = tag_predict_utils.segmentor(texts, labels)
            shutil.rmtree('./output')

        finally:
            os.chdir(self._work_path)

        if mode == 'cut':
            return cut
        elif mode == 'posseg':
            return posseg
        else:
            raise Exception("the mode must be cut or pooseg !")


class sentence(object):

    def __init__(self, model_dir):

        self._work_path = os.getcwd()
        self._module_path = os.path.dirname(__file__)
        self._model_dir = os.path.abspath(model_dir)

    def _data_process(self, document):

        try:
            os.chdir(self._module_path)
            if not os.path.exists('./output'):
                os.mkdir('./output')

            with open('./output/test.txt', 'w+', encoding='utf-8') as f:
                for sentence in document:
                    f.write(sentence)
                    f.write('\n')
        finally:
            os.chdir(self._work_path)

    def sentiment(self, document, full_msg=False):
        model_path = os.path.join(self._model_dir, 'sentiment_model/model')
        if type(document) is str:
            raise Exception('the document must be list or tuple')

        self._data_process(document)
        try:
            os.chdir(self._module_path)

            os.system('python sentence.py \
                          --task_name=classify \
                          --do_predict=true \
                          --data_dir=./output \
                          --vocab_file=./vocab.txt \
                          --bert_config_file=./bert_config.json \
                          --init_checkpoint={} \
                          --max_seq_length=128 \
                          --train_batch_size=32 \
                          --learning_rate=2e-5 \
                          --output_dir=./output'.format(model_path))

            lines = sentiment_predict_utils.get_data('./output/test_results.tsv')
            lines = sentiment_predict_utils.get_result(lines)
            results = sentiment_predict_utils.output(lines, full_msg)

            shutil.rmtree('./output')
            return results
        except Exception as e:
            print(e)
        finally:
            os.chdir(self._work_path)

    def pair(self, document, full_msg=False, model_name=None):
        if type(document) is str:
            raise Exception('the document must be list or tuple')
        model_path = os.path.join(self._model_dir, model_name)
        self._data_process(document)
        try:
            os.chdir(self._module_path)

            os.system('python sentence.py \
                          --task_name=pair \
                          --do_predict=true \
                          --data_dir=./output \
                          --vocab_file=./vocab.txt \
                          --bert_config_file=./bert_config.json \
                          --init_checkpoint={} \
                          --max_seq_length=128 \
                          --train_batch_size=32 \
                          --learning_rate=2e-5 \
                          --output_dir=./output'.format(model_path))

            lines = sentiment_predict_utils.get_data('./output/test_results.tsv')
            lines = sentiment_predict_utils.get_result(lines)
            results = sentiment_predict_utils.output(lines, full_msg)

            shutil.rmtree('./output')
            return results
        except Exception as e:
            print(e)
        finally:
            os.chdir(self._work_path)






# coding=utf-8
# Copyright 2018 The Google AI Language Team Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""BERT finetuning runner."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import codecs
import pickle
import collections
import os

import modeling
import tokenization
import tensorflow as tf



flags = tf.flags

FLAGS = flags.FLAGS

## Required parameters
flags.DEFINE_string(
    "data_dir", None,
    "The input data dir. Should contain the .tsv files (or other data files) "
    "for the task.")

flags.DEFINE_string(
    "bert_config_file", None,
    "The config json file corresponding to the pre-trained BERT model. "
    "This specifies the model architecture.")

flags.DEFINE_string("task_name", None, "The name of the task to train.")

flags.DEFINE_string("vocab_file", None,
                    "The vocabulary file that the BERT model was trained on.")

flags.DEFINE_string(
    "output_dir", None,
    "The output directory where the model checkpoints will be written.")

## Other parameters

flags.DEFINE_string(
    "init_checkpoint", None,
    "Initial checkpoint (usually from a pre-trained BERT model).")

flags.DEFINE_bool(
    "do_lower_case", True,
    "Whether to lower case the input text. Should be True for uncased "
    "models and False for cased models.")

flags.DEFINE_integer(
    "max_seq_length", 128,
    "The maximum total input sequence length after WordPiece tokenization. "
    "Sequences longer than this will be truncated, and sequences shorter "
    "than this will be padded.")

flags.DEFINE_bool("do_train", False, "Whether to run training.")

flags.DEFINE_bool("do_eval", False, "Whether to run eval on the dev set.")

flags.DEFINE_bool(
    "do_predict", False,
    "Whether to run the model in inference mode on the test set.")

flags.DEFINE_integer("train_batch_size", 32, "Total batch size for training.")

flags.DEFINE_integer("eval_batch_size", 8, "Total batch size for eval.")

flags.DEFINE_integer("predict_batch_size", 8, "Total batch size for predict.")

flags.DEFINE_float("learning_rate", 5e-5, "The initial learning rate for Adam.")

flags.DEFINE_float("num_train_epochs", 3.0,
                   "Total number of training epochs to perform.")

flags.DEFINE_float(
    "warmup_proportion", 0.1,
    "Proportion of training to perform linear learning rate warmup for. "
    "E.g., 0.1 = 10% of training.")

flags.DEFINE_integer("save_checkpoints_steps", 1000,
                     "How often to save the model checkpoint.")

flags.DEFINE_integer("iterations_per_loop", 1000,
                     "How many steps to make in each estimator call.")

flags.DEFINE_bool("use_tpu", False, "Whether to use TPU or GPU/CPU.")

tf.flags.DEFINE_string(
    "tpu_name", None,
    "The Cloud TPU to use for training. This should be either the name "
    "used when creating the Cloud TPU, or a grpc://ip.address.of.tpu:8470 "
    "url.")

tf.flags.DEFINE_string(
    "tpu_zone", None,
    "[Optional] GCE zone where the Cloud TPU is located in. If not "
    "specified, we will attempt to automatically detect the GCE project from "
    "metadata.")

tf.flags.DEFINE_string(
    "gcp_project", None,
    "[Optional] Project name for the Cloud TPU-enabled project. If not "
    "specified, we will attempt to automatically detect the GCE project from "
    "metadata.")

tf.flags.DEFINE_string("master", None, "[Optional] TensorFlow master URL.")

flags.DEFINE_integer(
    "num_tpu_cores", 8,
    "Only used if `use_tpu` is True. Total number of TPU cores to use.")


class InputExample(object):
    """A single training/test example for simple sequence classification."""

    def __init__(self, guid, text_a, text_b=None, label=None):
        """Constructs a InputExample.

        Args:
          guid: Unique id for the example.
          text_a: string. The untokenized text of the first sequence. For single
            sequence tasks, only this sequence must be specified.
          text_b: (Optional) string. The untokenized text of the second sequence.
            Only must be specified for sequence pair tasks.
          label: (Optional) string. The label of the example. This should be
            specified for train and dev examples, but not for test examples.
        """
        self.guid = guid
        self.text_a = text_a
        self.text_b = text_b
        self.label = label


class PaddingInputExample(object):
    """Fake example so the num input examples is a multiple of the batch size.

    When running eval/predict on the TPU, we need to pad the number of examples
    to be a multiple of the batch size, because the TPU requires a fixed batch
    size. The alternative is to drop the last batch, which is bad because it means
    the entire output data won't be generated.

    We use this class instead of `None` because treating `None` as padding
    battches could cause silent errors.
    """


class InputFeatures(object):
    """A single set of features of data."""

    def __init__(self,
                 input_ids,
                 input_mask,
                 segment_ids,
                 label_id, ):
        self.input_ids = input_ids
        self.input_mask = input_mask
        self.segment_ids = segment_ids
        self.label_id = label_id
        # self.is_real_example = is_real_example


class DataProcessor(object):
    """Base class for data converters for sequence classification data sets."""

    def get_train_examples(self, data_dir):
        """Gets a collection of `InputExample`s for the train set."""
        raise NotImplementedError()

    def get_dev_examples(self, data_dir):
        """Gets a collection of `InputExample`s for the dev set."""
        raise NotImplementedError()

    def get_test_examples(self, data_dir):
        """Gets a collection of `InputExample`s for prediction."""
        raise NotImplementedError()

    def get_labels(self):
        """Gets the list of labels for this data set."""
        raise NotImplementedError()

    # 序列标注任务数据读取
    @classmethod
    def _read_data_tagging(cls, input_file):
        """Reads a BIO data."""
        with open(input_file) as f:
            lines = []
            words = []
            labels = []
            for line in f:
                contends = line.strip()
                word = line.strip().split(' ')[0]
                label = line.strip().split(' ')[-1]
                if contends.startswith("-DOCSTART-"):
                    words.append('')
                    continue
                if len(contends) == 0:
                    l = ' '.join([label for label in labels if len(label) > 0])
                    w = ' '.join([word for word in words if len(word) > 0])
                    lines.append([l, w])
                    words = []
                    labels = []
                    continue
                words.append(word)
                labels.append(label)

            return lines

    @classmethod
    def _read_test_data_tagging(cls, input_file):
        with open(input_file,encoding='utf-8') as f:
            lines = []
            words = []
            labels = []
            for line in f:
                contends = line.strip()
                word = line.strip().split(' ')[0]
                label = '[CLS]'
                if contends.startswith("-DOCSTART-"):
                    words.append('')
                    continue
                if len(contends) == 0:
                    l = ' '.join([label for label in labels if len(label) > 0])
                    w = ' '.join([word for word in words if len(word) > 0])
                    lines.append([l, w])
                    words = []
                    labels = []
                    continue
                words.append(word)
                labels.append(label)
            return lines


class tagging_processor(DataProcessor):
    '''
    命名实体识别
    文件格式：txt
    数据格式：字 标注
    '''

    def get_train_examples(self, data_dir):
        return self._create_example(
            self._read_data_tagging(os.path.join(data_dir, "train.txt")), "train"
        )

    def get_dev_examples(self, data_dir):
        return self._create_example(
            self._read_data_tagging(os.path.join(data_dir, "dev.txt")), "dev"
        )

    def get_test_examples(self, data_dir):
        return self._create_example(
            self._read_test_data_tagging(os.path.join(data_dir, "test.txt")), "test")

    def get_labels(self):
        return ['B-NMC', 'B-RYT', 'S-DG', 'B-NHM', 'I-UGUO', 'B-NND', 'B-NSF', 'I-NS', 'B-NTU',
                'S-E', 'I-S', 'S-QV', 'S-NT', 'I-NT', 'I-NHM', 'S-CC', 'B-NBC', 'B-NTCH', 'B-NRF',
                'I-NRJ', 'B-NBA', 'I-PBEI', 'I-UYY', 'S-AG', 'I-AG', 'B-L', 'I-GC', 'I-W', 'I-VN',
                'I-NTCH', 'I-CC', 'I-Y', 'S-UDE2', 'B-QT', 'B-M', 'B-GM', 'B-NR', 'B-NIT', 'I-VYOU',
                'I-RYV', 'I-J', 'B-E', 'B-NTH', 'I-L', 'B-RZV', 'I-Z', 'I-NRF', 'B-NTS', 'B-NT',
                'B-RYV', 'S-VSHI', 'I-NTC', 'B-VN', 'I-DL', 'B-UYY', 'B-NZ', 'S-RZ', 'I-VI',
                'B-Q', 'I-MQ', 'S-RY', 'I-NA', 'S-ULIAN', 'B-N', 'I-O', 'B-J', 'I-D', 'S-VN',
                'B-K', 'I-N', 'S-NS', 'B-U', 'I-NIS', 'I-AN', 'I-B', 'B-AL', 'I-NSF', 'B-D',
                'S-C', 'B-RY', 'B-VD', 'I-K', 'I-RY', 'I-RZT', 'B-AN', 'B-VL', 'S-MQ', 'I-NG',
                'I-RZS', 'B-NG', 'S-P', 'S-VG', 'I-NNT', 'B-MQ', 'I-QT', 'S-Q', 'I-AL', 'S-VI',
                'S-R', 'I-P', 'I-ULS', 'S-NZ', 'S-NG', 'S-RZS', 'B-UDE1', 'I-RYS', 'B-RZS',
                'S-USUO', 'B-NA', 'I-NTS', 'S-V', 'S-AN', 'I-NBC', 'S-RR', 'B-DL', 'B-UZHE',
                'B-NS', 'I-GB', 'I-NTO', 'S-VF', 'I-AD', 'I-GP', 'I-Q', 'B-X', 'B-GB', 'B-NIS',
                'I-VL', 'B-RYS', 'I-UDE3', 'B-QV', 'I-USUO', 'I-BL', 'B-GP', 'I-U', 'B-ULS',
                'I-NR2', 'B-NR2', 'I-NTCF', 'B-T', 'S-TG', 'B-VF', 'I-NTCB', 'B-NTCF', 'I-TG',
                'B-P', 'B-F', 'I-NF', 'I-RYT', 'S-UYY', 'S-QT', 'I-X', 'S-UDE3', 'B-NTC', 'I-NBA',
                'I-I', 'B-VG', 'S-B', 'S-RZV', 'I-NND', 'I-C', 'S-D', 'S-Y', 'B-NF', 'B-NNT', 'S-W',
                'I-VX', 'S-PBA', 'B-AD', 'I-NZ', 'I-RZ', 'S-UZHE', 'I-RZV', 'B-VYOU', 'B-VI', 'B-NRJ',
                'I-NTU', 'S-UDENG', 'S-UZHI', 'I-VSHI', 'S-K', 'B-GI', 'S-N', 'I-QV', 'B-NTO', 'S-RYV',
                'S-O', 'S-PBEI', 'B-UDH', 'B-USUO', 'I-PBA', 'S-X', 'B-ULE', 'I-UZHE', 'B-NHD', 'I-M',
                'B-RZT', 'I-T', 'S-A', 'S-VYOU', 'B-Z', 'I-R', 'B-VSHI', 'B-PBEI', 'I-GM', 'B-BL',
                'I-NIT', 'S-AD', 'B-O', 'I-V', 'I-F', 'I-NR', 'B-S', 'I-NMC', 'I-UDE1', 'S-NF', 'B-TG',
                'I-RR', 'B-B', 'S-MG', 'B-RR', 'B-GC', 'B-V', 'B-RZ', 'I-UDENG', 'I-GI', 'B-PBA', 'S-M',
                'B-CC', 'B-NTCB', 'I-E', 'I-UDH', 'I-A', 'I-VD', 'B-UDENG', 'B-Y', 'S-UGUO', 'B-W',
                'S-RG', 'B-AG', 'I-VG', 'S-F', 'B-VX', 'S-ULE', 'S-UDE1', 'B-GG', 'B-I', 'B-C',
                'I-ULE', 'I-GG', 'B-UDE3', 'B-UGUO', 'B-R', 'B-A', 'S-NHD', 'I-VF', 'I-NHD',
                'I-NTH', '[CLS]', '[SEP]']

    def _create_example(self, lines, set_type):
        examples = []
        for (i, line) in enumerate(lines):
            guid = "%s-%s" % (set_type, i)
            text = tokenization.convert_to_unicode(line[1])
            label = tokenization.convert_to_unicode(line[0])
            examples.append(InputExample(guid=guid, text_a=text, label=label))
        return examples


class NER_processor(DataProcessor):
    '''
    命名实体识别
    文件格式：txt
    数据格式：字 标注
    '''

    def get_train_examples(self, data_dir):
        return self._create_example(
            self._read_data_tagging(os.path.join(data_dir, "train.txt")), "train"
        )

    def get_dev_examples(self, data_dir):
        return self._create_example(
            self._read_data_tagging(os.path.join(data_dir, "dev.txt")), "dev"
        )

    def get_test_examples(self, data_dir):
        return self._create_example(
            self._read_test_data_tagging(os.path.join(data_dir, "test.txt")), "test")

    def get_labels(self):
        return ["O", "B-PER", "I-PER", "B-ORG", "I-ORG", "B-LOC", "I-LOC", "X", "[CLS]", "[SEP]"]

    def _create_example(self, lines, set_type):
        examples = []
        for (i, line) in enumerate(lines):
            guid = "%s-%s" % (set_type, i)
            text = tokenization.convert_to_unicode(line[1])
            label = tokenization.convert_to_unicode(line[0])
            examples.append(InputExample(guid=guid, text_a=text, label=label))
        return examples


def write_tokens(tokens, mode):
    """
    将序列解析结果写入到文件中
    只在mode=test的时候启用
    :param tokens:
    :param mode:
    :return:
    """
    if mode == "test":
        path = os.path.join(FLAGS.output_dir, "token_" + mode + ".txt")
        wf = codecs.open(path, 'a', encoding='utf-8')
        for token in tokens:
            if token != "**NULL**":
                wf.write(token + '\n')
        wf.close()


def convert_single_example(ex_index, example, label_list, max_seq_length,
                           tokenizer,
                           mode=None):
    label_map = {}
    ### 序列标注问题数据解析方式

    for (i, label) in enumerate(label_list, 1):
        label_map[label] = i

    # # 保存label->index 的map
    # if not os.path.exists(os.path.join(FLAGS.output_dir, 'label2id.pkl')):
    #     with codecs.open(os.path.join(FLAGS.output_dir, 'label2id.pkl'), 'wb') as w:
    #         pickle.dump(label_map, w)

    textlist = example.text_a.split(' ')
    labellist = example.label.split(' ')
    tokens = []
    labels = []
    for i, word in enumerate(textlist):
        # 分词，如果是中文，就是分字
        token = tokenizer.tokenize(word)
        tokens.extend(token)
        label_1 = labellist[i]
        for m in range(len(token)):
            if m == 0:
                labels.append(label_1)
            else:  # 一般不会出现else
                labels.append("X")
    # 序列截断
    if len(tokens) >= max_seq_length - 1:
        tokens = tokens[0:(max_seq_length - 2)]  # -2 的原因是因为序列需要加一个句首和句尾标志
        labels = labels[0:(max_seq_length - 2)]

    ntokens = []
    segment_ids = []
    label_ids = []
    ntokens.append("[CLS]")  # 句子开始设置CLS 标志
    segment_ids.append(0)
    label_ids.append(label_map["[CLS]"])  # O OR CLS 没有任何影响，不过我觉得O 会减少标签个数,不过句首和句尾使用不同的标志来标注，使用CLS也没毛病
    for i, token in enumerate(tokens):
        ntokens.append(token)
        segment_ids.append(0)
        label_ids.append(label_map[labels[i]])
    ntokens.append("[SEP]")  # 句尾添加[SEP] 标志
    segment_ids.append(0)
    label_ids.append(label_map["[SEP]"])
    input_ids = tokenizer.convert_tokens_to_ids(ntokens)  # 将序列中的字(ntokens)转化为ID形式
    input_mask = [1] * len(input_ids)
    # padding
    while len(input_ids) < max_seq_length:
        input_ids.append(0)
        input_mask.append(0)
        segment_ids.append(0)
        label_ids.append(0)
        ntokens.append("**NULL**")

    assert len(input_ids) == max_seq_length
    assert len(input_mask) == max_seq_length
    assert len(segment_ids) == max_seq_length
    assert len(label_ids) == max_seq_length

    # 结构化为一个类
    feature = InputFeatures(
        input_ids=input_ids,
        input_mask=input_mask,
        segment_ids=segment_ids,
        label_id=label_ids,
    )

    write_tokens(ntokens, mode)
    return feature


def file_based_convert_examples_to_features(
        examples, label_list, max_seq_length, tokenizer, output_file,
        mode=None):
    """Convert a set of `InputExample`s to a TFRecord file."""

    writer = tf.python_io.TFRecordWriter(output_file)

    for (ex_index, example) in enumerate(examples):
        if ex_index % 10000 == 0:
            tf.logging.info("Writing example %d of %d" % (ex_index, len(examples)))

        feature = convert_single_example(ex_index, example, label_list,
                                         max_seq_length, tokenizer, mode)

        def create_int_feature(values):
            f = tf.train.Feature(int64_list=tf.train.Int64List(value=list(values)))
            return f

        features = collections.OrderedDict()
        features["input_ids"] = create_int_feature(feature.input_ids)
        features["input_mask"] = create_int_feature(feature.input_mask)
        features["segment_ids"] = create_int_feature(feature.segment_ids)
        features["label_ids"] = create_int_feature(feature.label_id)
        tf_example = tf.train.Example(features=tf.train.Features(feature=features))
        writer.write(tf_example.SerializeToString())
    writer.close()


def file_based_input_fn_builder(input_file, seq_length, is_training,
                                drop_remainder):
    """Creates an `input_fn` closure to be passed to TPUEstimator."""
    name_to_features = {
        "input_ids": tf.FixedLenFeature([seq_length], tf.int64),
        "input_mask": tf.FixedLenFeature([seq_length], tf.int64),
        "segment_ids": tf.FixedLenFeature([seq_length], tf.int64),
        "label_ids": tf.FixedLenFeature([seq_length], tf.int64),
    }


    def _decode_record(record, name_to_features):
        """Decodes a record to a TensorFlow example."""
        example = tf.parse_single_example(record, name_to_features)

        # tf.Example only supports tf.int64, but the TPU only supports tf.int32.
        # So cast all int64 to int32.
        for name in list(example.keys()):
            t = example[name]
            if t.dtype == tf.int64:
                t = tf.to_int32(t)
            example[name] = t

        return example

    def input_fn(params):
        """The actual input function."""
        batch_size = params["batch_size"]
        # For training, we want a lot of parallel reading and shuffling.
        # For eval, we want no shuffling and parallel reading doesn't matter.
        d = tf.data.TFRecordDataset(input_file)
        if is_training:
            d = d.repeat()
            d = d.shuffle(buffer_size=100)

        d = d.apply(
            tf.contrib.data.map_and_batch(
                lambda record: _decode_record(record, name_to_features),
                batch_size=batch_size,
                drop_remainder=drop_remainder))

        return d

    return input_fn


def _truncate_seq_pair(tokens_a, tokens_b, max_length):
    """Truncates a sequence pair in place to the maximum length."""

    # This is a simple heuristic which will always truncate the longer sequence
    # one token at a time. This makes more sense than truncating an equal percent
    # of tokens from each, since if one sequence is very short then each token
    # that's truncated likely contains more information than a longer sequence.
    while True:
        total_length = len(tokens_a) + len(tokens_b)
        if total_length <= max_length:
            break
        if len(tokens_a) > len(tokens_b):
            tokens_a.pop()
        else:
            tokens_b.pop()


def create_model(bert_config, is_training, input_ids, input_mask, segment_ids,
                 labels, num_labels, use_one_hot_embeddings):
    # 使用数据加载BertModel,获取对应的字embedding
    # 即特征提取层
    model = modeling.BertModel(
        config=bert_config,
        is_training=is_training,
        input_ids=input_ids,
        input_mask=input_mask,
        token_type_ids=segment_ids,
        use_one_hot_embeddings=use_one_hot_embeddings)

    # 对模型进行fine_tuning
    # 要实现对模型的改造与使用，全部在这个函数中进行
    # 获取对应的embedding 输入数据[batch_size, seq_length, embedding_size]

    output_layer = model.get_sequence_output()

    hidden_size = output_layer.shape[-1].value

    output_weight = tf.get_variable(
        "output_weights", [num_labels, hidden_size],
        initializer=tf.truncated_normal_initializer(stddev=0.02)
    )
    output_bias = tf.get_variable(
        "output_bias", [num_labels], initializer=tf.zeros_initializer()
    )
    with tf.variable_scope("loss"):
        if is_training:
            output_layer = tf.nn.dropout(output_layer, keep_prob=0.9)

        # 将词性标注问题看做是对每个位置的词的分类问题
        output_layer = tf.reshape(output_layer, [-1, hidden_size])
        logits = tf.matmul(output_layer, output_weight, transpose_b=True)
        logits = tf.nn.bias_add(logits, output_bias)
        logits = tf.reshape(logits, [-1, FLAGS.max_seq_length, num_labels])

        log_probs = tf.nn.log_softmax(logits, axis=-1)
        one_hot_labels = tf.one_hot(labels, depth=num_labels, dtype=tf.float32)
        per_example_loss = -tf.reduce_sum(one_hot_labels * log_probs, axis=-1)
        loss = tf.reduce_sum(per_example_loss)

        probabilities = tf.nn.softmax(logits, axis=-1)
        predict = tf.argmax(probabilities, axis=-1)

        return (loss, per_example_loss, logits, predict)

def model_fn_builder(bert_config, num_labels, init_checkpoint, learning_rate,
                     num_train_steps, num_warmup_steps, use_tpu,
                     use_one_hot_embeddings):
    """Returns `model_fn` closure for TPUEstimator."""

    def model_fn(features, labels, mode, params):  # pylint: disable=unused-argument
        """The `model_fn` for TPUEstimator."""

        input_ids = features["input_ids"]
        input_mask = features["input_mask"]
        segment_ids = features["segment_ids"]
        label_ids = features["label_ids"]

        is_training = (mode == tf.estimator.ModeKeys.TRAIN)
        # 使用参数构建模型,input_idx 就是输入的样本idx表示，label_ids 就是标签的idx表示
        (total_loss, per_example_loss, logits, predicts) = create_model(
            bert_config, is_training, input_ids, input_mask, segment_ids, label_ids,
            num_labels, use_one_hot_embeddings)

        tvars = tf.trainable_variables()
        initialized_variable_names = {}
        scaffold_fn = None
        # 加载BERT模型
        if init_checkpoint:
            (assignment_map, initialized_variable_names
             ) = modeling.get_assignment_map_from_checkpoint(tvars, init_checkpoint)
            tf.train.init_from_checkpoint(init_checkpoint, assignment_map)
            if use_tpu:

                def tpu_scaffold():
                    tf.train.init_from_checkpoint(init_checkpoint, assignment_map)
                    return tf.train.Scaffold()

                scaffold_fn = tpu_scaffold
            else:
                tf.train.init_from_checkpoint(init_checkpoint, assignment_map)

        output_spec = None

        output_spec = tf.contrib.tpu.TPUEstimatorSpec(
            mode=mode, predictions=predicts, scaffold_fn=scaffold_fn
        )

        return output_spec

    return model_fn


# This function is not used by this file but is still used by the Colab and
# people who depend on it.
def input_fn_builder(features, seq_length, is_training, drop_remainder):
    """Creates an `input_fn` closure to be passed to TPUEstimator."""

    all_input_ids = []
    all_input_mask = []
    all_segment_ids = []
    all_label_ids = []

    for feature in features:
        all_input_ids.append(feature.input_ids)
        all_input_mask.append(feature.input_mask)
        all_segment_ids.append(feature.segment_ids)
        all_label_ids.append(feature.label_id)

    def input_fn(params):
        """The actual input function."""
        batch_size = params["batch_size"]

        num_examples = len(features)

        # This is for demo purposes and does NOT scale to large data sets. We do
        # not use Dataset.from_generator() because that uses tf.py_func which is
        # not TPU compatible. The right way to load data is with TFRecordReader.
        d = tf.data.Dataset.from_tensor_slices({
            "input_ids":
                tf.constant(
                    all_input_ids, shape=[num_examples, seq_length],
                    dtype=tf.int32),
            "input_mask":
                tf.constant(
                    all_input_mask,
                    shape=[num_examples, seq_length],
                    dtype=tf.int32),
            "segment_ids":
                tf.constant(
                    all_segment_ids,
                    shape=[num_examples, seq_length],
                    dtype=tf.int32),
            "label_ids":
                tf.constant(all_label_ids, shape=[num_examples], dtype=tf.int32),
        })

        if is_training:
            d = d.repeat()
            d = d.shuffle(buffer_size=100)

        d = d.batch(batch_size=batch_size, drop_remainder=drop_remainder)
        return d

    return input_fn

def main(_):
    tf.logging.set_verbosity('WARN')

    processors = {
        "ner": NER_processor,
        "tag": tagging_processor
    }

    tokenization.validate_case_matches_checkpoint(FLAGS.do_lower_case,
                                                  FLAGS.init_checkpoint)


    bert_config = modeling.BertConfig.from_json_file(FLAGS.bert_config_file)

    if FLAGS.max_seq_length > bert_config.max_position_embeddings:
        raise ValueError(
            "Cannot use sequence length %d because the BERT model "
            "was only trained up to sequence length %d" %
            (FLAGS.max_seq_length, bert_config.max_position_embeddings))

    tf.gfile.MakeDirs(FLAGS.output_dir)

    task_name = FLAGS.task_name.lower()

    processor = processors[task_name]()

    label_list = processor.get_labels()

    tokenizer = tokenization.FullTokenizer(
        vocab_file=FLAGS.vocab_file, do_lower_case=FLAGS.do_lower_case)

    tpu_cluster_resolver = None
    if FLAGS.use_tpu and FLAGS.tpu_name:
        tpu_cluster_resolver = tf.contrib.cluster_resolver.TPUClusterResolver(
            FLAGS.tpu_name, zone=FLAGS.tpu_zone, project=FLAGS.gcp_project)

    is_per_host = tf.contrib.tpu.InputPipelineConfig.PER_HOST_V2
    run_config = tf.contrib.tpu.RunConfig(
        cluster=tpu_cluster_resolver,
        master=FLAGS.master,
        model_dir=FLAGS.output_dir,
        save_checkpoints_steps=FLAGS.save_checkpoints_steps,
        tpu_config=tf.contrib.tpu.TPUConfig(
            iterations_per_loop=FLAGS.iterations_per_loop,
            num_shards=FLAGS.num_tpu_cores,
            per_host_input_for_training=is_per_host))

    train_examples = None
    num_train_steps = None
    num_warmup_steps = None


    ### 对于词性标注问题，注意num_label的值
    model_fn = model_fn_builder(
        bert_config=bert_config,
        num_labels=len(label_list) + 1,
        init_checkpoint=FLAGS.init_checkpoint,
        learning_rate=FLAGS.learning_rate,
        num_train_steps=num_train_steps,
        num_warmup_steps=num_warmup_steps,
        use_tpu=FLAGS.use_tpu,
        use_one_hot_embeddings=FLAGS.use_tpu)

    # If TPU is not available, this will fall back to normal Estimator on CPU
    # or GPU.
    estimator = tf.contrib.tpu.TPUEstimator(
        use_tpu=FLAGS.use_tpu,
        model_fn=model_fn,
        config=run_config,
        train_batch_size=FLAGS.train_batch_size,
        eval_batch_size=FLAGS.eval_batch_size,
        predict_batch_size=FLAGS.predict_batch_size)


    if FLAGS.do_predict:
        id2label={}
        token_path = os.path.join(FLAGS.output_dir, "token_test.txt")
        if task_name == 'tag':
            with open(os.path.join('./', 'pos_tag.pkl'), 'rb') as rf:
                label2id = pickle.load(rf)
                id2label = {value: key for key, value in label2id.items()}
        if task_name == 'ner':
            with open(os.path.join('./', 'ner.pkl'), 'rb') as rf:
                label2id = pickle.load(rf)
                id2label = {value: key for key, value in label2id.items()}
        if os.path.exists(token_path):
            os.remove(token_path)

        predict_examples = processor.get_test_examples(FLAGS.data_dir)
        predict_file = os.path.join(FLAGS.output_dir, "predict.tf_record")
        file_based_convert_examples_to_features(predict_examples, label_list,
                                                FLAGS.max_seq_length, tokenizer,
                                                predict_file, mode="test")

        tf.logging.info("***** Running prediction*****")
        tf.logging.info("  Num examples = %d", len(predict_examples))
        tf.logging.info("  Batch size = %d", FLAGS.predict_batch_size)
        if FLAGS.use_tpu:
            # Warning: According to tpu_estimator.py Prediction on TPU is an
            # experimental feature and hence not supported here
            raise ValueError("Prediction in TPU not supported")
        predict_drop_remainder = True if FLAGS.use_tpu else False
        predict_input_fn = file_based_input_fn_builder(
            input_file=predict_file,
            seq_length=FLAGS.max_seq_length,
            is_training=False,
            drop_remainder=predict_drop_remainder)

        result = estimator.predict(input_fn=predict_input_fn)
        output_predict_file = os.path.join(FLAGS.output_dir, "label_test.txt")
        with open(output_predict_file, 'w') as writer:
            for prediction in result:
                output_line = "\n".join(id2label[id] for id in prediction if id != 0) + "\n"
                writer.write(output_line)

if __name__ == "__main__":
    flags.mark_flag_as_required("data_dir")
    flags.mark_flag_as_required("task_name")
    flags.mark_flag_as_required("vocab_file")
    flags.mark_flag_as_required("bert_config_file")
    flags.mark_flag_as_required("output_dir")
    tf.app.run()



import tensorflow as tf

flags=tf.flags
FLAGS=flags.FLAGS

flags.DEFINE_string('input_file',default=None,help="input model file")
flags.DEFINE_string('output_file',None,help="output model file")

sess = tf.Session()
imported_meta = tf.train.import_meta_graph('{}.meta'.format(FLAGS.input_file))
imported_meta.restore(sess, FLAGS.input_file)
my_vars = []
for var in tf.global_variables():
    if 'adam_v' not in var.name and 'adam_m' not in var.name:
        my_vars.append(var)
saver = tf.train.Saver(my_vars)
saver.save(sess, FLAGS.output_file)



# -*- coding: utf-8 -*-

"""
 @Time    : 2019/3/12
 @Author  : tudou (upczyxl@163.com)
 @File    : __init__.py.py
"""
import os
import sys

current_dir = os.path.abspath(os.path.dirname(__file__))
sys.path.append(current_dir)
sys.path.append("..")


import os

class tools(object):
    def __init__(self):
        self.work_path=os.getcwd()
        self.module_path=os.path.dirname(__file__)

        self.path=os.path.abspath(self.work_path)

    def posseg_data(self,input_dir,output_file,combine=False,
                    single_line=False,log=False,max_len=128):
        '''
        将使用词性标注的文件转换为用BIS分块标记的文件。
        :param input_dir: 指定存放语料库的文件夹，程序将会递归查找目录下的文件。
        :param output_dir: 指定标记好的文件的输出路径。
        :param combine: 是否组装为一个文件
        :param single_line: 是否为单行模式
        :param log: 是否打印进度条
        :param max_len: 处理后的最大语句长度（将原句子按标点符号断句，若断句后的长度仍比最大长度长，将忽略
        :return:
        '''
        input_dir1=os.path.abspath(input_dir)
        output=os.path.abspath(output_file)
        try:
            os.chdir(self.module_path)
            os.system("python convert_to_tagdata.py {} {}".format(input_dir1,output))
        finally:
            os.chdir(self.work_path)

    def compress_model(self,input_file,output_file):
        '''
        参数为文件名称而不是目录名称
        :param input_file:
        :param output_file:
        :return:
        '''
        input1=os.path.abspath(input_file)
        output=os.path.abspath(output_file)
        print(output)

        try:
            os.chdir(self.module_path)
            os.system("python compress_ckpt.py --input_file={} --output_file={}".format(input1,output))
        finally:
            os.chdir(self.work_path)






def get_data(file):
    with open(file, 'r', encoding='utf-8') as f:
        lines = f.readlines()

    return lines


def proecess_data(texts):
    lines = []
    setence = []
    for word in texts:
        if word.strip() == '[CLS]':
            continue
        if word.strip() == '[SEP]':
            lines.append(setence)
            setence = []
            continue
        setence.append(word.strip())

    return lines


def recognized_entity(texts, labels):

    result=[]

    for text,label in zip(texts,labels):
        p,l,o=[],[],[]
        p_,l_,o_=[],[],[]
        i=0
        while (i<len(text)):

            if label[i].startswith('O'):
                '''BO IO'''
                if i !=0:
                    if label[i-1].endswith('PER'):
                        p_.append(''.join(p))
                        p = []
                    if label[i-1].endswith('LOC'):
                        l_.append(''.join(l))
                        l = []
                    if label[i-1].endswith('ORG'):
                        o_.append(''.join(o))
                        o = []

            if label[i].startswith(('B')):
                if i==0:
                    '''B'''
                    if label[i].endswith('PER'):
                        p.extend(text[i])
                    if label[i].endswith('LOC'):
                        l.extend(text[i])
                    if label[i].endswith('ORG'):
                        o.extend(text[i])
                else:
                    if label[i-1].startswith('O'):
                        '''OB'''
                        if label[i].endswith('PER'):
                            p.extend(text[i])
                        if label[i].endswith('LOC'):
                            l.extend(text[i])
                        if label[i].endswith('ORG'):
                            o.extend(text[i])
                    else:
                        '''IB BB'''
                        if label[i-1].endswith('PER'):
                            p_.append(''.join(p))
                            p=[]
                            if label[i].endswith('PER'):
                                p.extend(text[i])
                            if label[i].endswith('LOC'):
                                l.extend(text[i])
                            if label[i].endswith('ORG'):
                                o.extend(text[i])
                        if label[i-1].endswith('LOC'):
                            l_.append(''.join(l))
                            l = []
                            if label[i].endswith('PER'):
                                p.extend(text[i])
                            if label[i].endswith('LOC'):
                                l.extend(text[i])
                            if label[i].endswith('ORG'):
                                o.extend(text[i])
                        if label[i-1].endswith('ORG'):
                            o_.append(''.join(o))
                            o = []
                            if label[i].endswith('PER'):
                                p.extend(text[i])
                            if label[i].endswith('LOC'):
                                l.extend(text[i])
                            if label[i].endswith('ORG'):
                                o.extend(text[i])
            if label[i].startswith('I'):
                '''OI BI II'''
                if label[i].endswith('PER'):
                    p.extend(text[i])
                if label[i].endswith('LOC'):
                    l.extend(text[i])
                if label[i].endswith('ORG'):
                    o.extend(text[i])
            i+=1

        result.append((p_,l_,o_))


    return result


# def test():
#     text_file = "../predict_bert_function/output/token_test.txt"
#     label_file = "../predict_bert_function/output/label_test.txt"
#     texts = get_data(text_file)
#     labels = get_data(label_file)
#     texts = proecess_data(texts)
#     labels = proecess_data(labels)
#
#     print(recognized_entity(texts, labels))
#
#
# if __name__ == '__main__':
#     test()



import math


def get_data(file):
    with open(file, 'r', encoding='utf-8') as f:
        lines = f.readlines()

    return lines


def get_result(lines):
    results = []
    for line in lines:
        results.append(line.split())

    return results


def arg_result(datas):
    r = []
    for data in datas:
        r.append(float(data))

    max_ = max(r)
    index = r.index(max_)

    if index == 0:
        return ('Neutral:', max_)

    elif index == 1:
        return ('Positive:', max_)

    else:
        return ('Negative:', max_)


def full_result(datas):
    r = []
    dict = {0: 'Neutral', 1: 'Positive', 2: 'Negative'}
    for i, data in enumerate(datas):
        r.append((dict[i], float(data)))

    return r

def output(texts,full=False):
    r=[]
    for text in texts:
        if full:
            r.append(full_result(text))
        else:
            r.append(arg_result(text))

    return r

# def test():
#     lines = get_data('../predict_bert_function/output/test_results.tsv')
#     r = get_result(lines)
#     print(output(r))
#
#
# if __name__ == '__main__':
#     test()



import os
import re
import argparse

def print_process(process):
    num_processed = int(30 * process)
    num_unprocessed = 30 - num_processed
    print(
        f"{''.join(['['] + ['='] * num_processed + ['>'] + [' '] * num_unprocessed + [']'])}, {(process * 100):.2f} %")


def convert_to_bis(source_dir, target_path, log=False, combine=False, single_line=True):
    print("Converting...")
    for root, dirs, files in os.walk(source_dir):
        total = len(files)
        tgt_dir = target_path + root[len(source_dir):]

        print(tgt_dir)
        for index, name in enumerate(files):
            file = os.path.join(root, name)
            bises = process_file(file)
            if combine:
                _save_bises(bises, target_path, write_mode='a', single_line=single_line)
            else:
                os.makedirs(tgt_dir, exist_ok=True)
                _save_bises(bises, os.path.join(tgt_dir, name), single_line=single_line)
            if log:
                print_process((index + 1) / total)
    print("All converted")


def _save_bises(bises, path, write_mode='w+', single_line=True):
    with open(path, mode=write_mode, encoding='UTF-8') as f:
        if single_line:
            for bis in bises:
                sent, tags = [], []
                for char, tag in bis:
                    sent.append(char)
                    tags.append(tag)
                sent = ' '.join(sent)
                tags = ' '.join(tags)
                f.write(sent + "\t" + tags)
                f.write('\n')
        else:
            for bis in bises:
                for char, tag in bis:
                    f.write(char + "\t" + tag + "\n")
                f.write("\n")


def process_file(file):
    with open(file, 'r', encoding='UTF-8') as f:
        text = f.readlines()
        bises = _parse_text(text)
    return bises


def _parse_text(text: list):
    bises = []
    for line in text:
        # remove POS tag
        line, _ = re.subn('\\n', '', line)
        if line == '' or line == '\n':
            continue
        words = re.split('\s+', line)

        if len(words) > MAX_LEN_SIZE:
            texts = re.split('[。？！，.?!,]/w', line)
            if len(min(texts, key=len)) > MAX_LEN_SIZE:
                continue
            bises.extend(_parse_text(texts))
        else:
            bises.append(_tag(words))
    return bises


def _tag(words):
    """
    给指定的一行文本打上BIS标签
    :param line: 文本行
    :return:
    """
    bis = []
    # words = list(map(list, words))
    pre_word = None
    for word in words:
        pos_t = None
        tokens = word.split('/')
        if len(tokens) == 2:
            word, pos = tokens
        elif len(tokens) == 3:
            word, pos_t, pos = tokens
        else:
            continue

        word = list(word)
        pos = pos.upper()

        if len(word) == 0:
            continue
        if word[0] == '[':
            pre_word = word
            continue
        if pre_word is not None:
            pre_word += word
            if pos_t is None:
                continue
            elif pos_t[-1] != ']':
                continue
            else:
                word = pre_word[1:]
                pre_word = None

        if len(word) == 1:
            bis.append((word[0], 'S-' + pos))
        else:
            for i, char in enumerate(word):
                if i == 0:
                    bis.append((char, 'B-' + pos))
                else:
                    bis.append((char, 'I-' + pos))
    # bis.append(('\n', 'O'))
    return bis


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="将使用词性标注的文件转换为用BIS分块标记的文件。")
    parser.add_argument("corups_dir", type=str, help="指定存放语料库的文件夹，程序将会递归查找目录下的文件。")
    parser.add_argument("output_path", type=str, default='.', help="指定标记好的文件的输出路径。")
    parser.add_argument("-c", "--combine", help="是否组装为一个文件", default=True, type=bool)
    parser.add_argument("-s", "--single_line", help="是否为单行模式", default=False, type=bool)
    parser.add_argument("--log", help="是否打印进度条", default=True, type=bool)
    parser.add_argument("--max_len", help="处理后的最大语句长度（将原句子按标点符号断句，若断句后的长度仍比最大长度长，将忽略",
                        default=128, type=int)
    args = parser.parse_args()
    MAX_LEN_SIZE = args.max_len

    convert_to_bis(args.corups_dir, args.output_path, args.log, args.combine, args.single_line)


def get_labels(file):
    labels=set()
    with open(file,'r',encoding='utf-8') as f:
        for line in f:
            try:
                labels.add(line.split()[1])
            except:
                pass

    return list(labels)

if __name__ == '__main__':
    labels=get_labels('../../datasets/train.txt')
    print(labels)


import tqdm
import argparse


def read_file(file):
    with open(file,'r',encoding='utf-8') as f:
        text=f.readlines()

    return text


def convert(input_file,output_file):
    texts=read_file(file=input_file)
    with open(output_file,'w+',encoding='utf-8') as f:
        for i in tqdm.tqdm(range(len(texts))):
            text, tagged = texts[i].split('\t')
            words = text.strip().split(' ')
            tags = tagged.strip().split(' ')

            for word, tag in zip(words, tags):
                f.write(word+' '+tag+'\n')
            f.write('\n')
if __name__ == '__main__':
    parser=argparse.ArgumentParser(description="将分词标注数据转换为bert可以读取的数据格式")
    parser.add_argument('input_file',type=str,help="输入文件")
    parser.add_argument('output_file',type=str,help="输出文件")

    args=parser.parse_args()
    convert(args.input_file,args.output_file)



from setuptools import setup,find_packages

setup(name='tudou',
      version='2.0.1',
      description='A Chinese NLP tools based Google-bert',
      long_description=open('README.md').read(),
      author='fennuDetudou',
      author_email='upczyxl@163.com',
      packages=find_packages(),
      # include_package_data=True,
      license='MIT',
      package_data={
            # If any package contains *.txt or *.rst files, include them:
            '': ['*.txt', '*.rst','*.pkl','*.json']
      },
      keywords='bert nlp ner NER named entity recognition tensorflow machine learning sentence encoding embedding pos tag sentiment judge',
      )




def get_data(file):
    with open(file, 'r', encoding='utf-8') as f:
        lines = f.readlines()

    return lines


def proecess_data(texts):
    line = []
    setence = []
    lines=[]
    for word in texts:
        if word.strip() == '[CLS]':
            continue
        if word.strip() == '[SEP]':
            line.append(setence)
            setence = []
            continue
        setence.append(word.strip())

    return line


def segmentor(texts, labels):

    cut_result=[]
    posseg_result=[]


    for text,label in zip(texts,labels):
        text_ = []
        label_ = []

        t, l = [], []
        i = 0
        while (i < len(text)):
            if label[i].startswith(('B', 'S')):
                text_.append(t)
                label_.append(l)
                t = []
                l = []
                t.extend(text[i])
                l.extend(label[i].split('-')[-1])
            elif label[i].startswith('I'):
                t.extend(text[i])
            i += 1
        text_.append(t)
        label_.append(l)

        t_result = text_[1:]
        l_result = label_[1:]

        sentence=[]
        posseg=[]
        for i, word in enumerate(t_result):
            sentence.append(''.join(word))
            posseg.append(''.join(l_result[i]))

        cut_result.append(sentence)
        posseg_result.append(posseg)

    results=[]
    for c,p in zip(cut_result,posseg_result):
        results.append(list(zip(c,p)))

    return cut_result,results


# def test():
#     text_file = '../predict_bert_function/output/token_test.txt'
#     label_file = '../predict_bert_function/output/label_test.txt'
#     texts = get_data(text_file)
#     label = get_data(label_file)
#     texts = proecess_data(texts)
#     labels = proecess_data(label)
#
#     a = segmentor(texts, labels)
#     print(a)
#
#
# if __name__ == '__main__':
#     test()



# -*- coding: utf-8 -*-

"""
 @Time    : 2019/3/12
 @Author  : tudou (upczyxl@163.com)
 @File    : __init__.py.py
"""

import sys
import os

current_dir = os.path.abspath(os.path.dirname(__file__))
sys.path.append(current_dir)
sys.path.append("..")
sys.path.append(os.path.join(current_dir,'models'))
sys.path.append(os.path.join(current_dir,'tools'))

from .models import train
from .models import predict
from .tools import utils
